require 'gui'
require 'keybinding'
require 'physics'
require 'character_functions'
require 'character_behaviour'
require 'projectile_behaviour'
require 'items'
require 'texts'
require 'item_documentation'
require 'character_documentation'
require 'charset_creation'
require 'animations'

function love.load()

	math.randomseed( os.time() )
	--key values
	TileSize = 32
	TilesetX, TilesetY = 512, 1024
	SizeX, SizeY = 200, 200
	ScreenSizeX, ScreenSizeY = 32, 20+1
	
	--scrolling
	OffsetX, OffsetY = 9,13					--IN TILE UNITS!!
	SmoothOffsetX, SmoothOffsetY = 0,0 		-- in pixels
	OffsetBorder = 8	
	Scrolling_Value = 1						--how many tiles does it jump?
	
	--charset sizes
	CharX = 32
	CharY = CharX*2
	CharFullX, CharFullY = CharX*6, CharY*4
	
	--general variables
	x,y,z,i,j,k = 0,0,0,0,0,0
	FPS = 30	
	
	--important sets	
	--character properties
	Alliances = {
	"United Forces", "Streetknight", "Church", "Red Fist", "Justice Empire", "Jade Legion",
	"Golddigger", "Khali", "Shadow Army", "Cloudan", "Moebius", "Android", "Not alliance-specific" }
	
	Character_Types = {
	"Test Puppet", "Elric Test", "Civilian", "Trader"}
	
	Character_States = {
	"Doing nothing", "Patrolling", "Turning around", "Walking"}
	
	Item_Types = {
	"Weapon", "Supply", "Everyday item",}
	
	--technical sets
	MovSet = { "up", "down", "left", "right" }					--the four directions
	SelectorPositions = {64,168,272,376,584,688,792,896}		--calibration for hotbar icon appearance
	BrokenProjectiles = {}
	
	--game variables
	game_clock = 0
	Item_Variety = 1000			--number of possible items in the game!
	
	GameOver = false 			--true if player is killed
	Immortal = false				--player health doesn't change
	
	--GUI
	MouseOnField = true
	
	--menus
	Inventory = false			--true if inventory is open
	InvX, InvY = 64, 64			--inventory menu variables
	InvListStart = 0			--where item listing begins
	InvColumnLength = 8			--length of a column
	InvWeight = 0				--sum of weight of inventory items
	InvColumnNum = 6			--number of columns on screen
	InvRowDist = 56
	InvColDist = 96
	InvSelectedItemID = 1		--ID of the item type that can be find at mouse position in inventory menu
	
	DialogueActive = false		--true if player is in a conversation
	DialogX, DialogY = 64, 64	--dialogue window variables	
	Dial_Random_Text = 0		--used in case of randomly generated character speak
	
	--mouse stuff
	MouseX, MouseY, MouseTileX, MouseTileY = 0,0,1,1

	--HUD elements
	Picset = {}
	Picset = {
		love.graphics.newImage('GUI/HUD_A.png'),
		love.graphics.newImage('GUI/SELECT.png'),	
		love.graphics.newImage('GUI/HUD_B.png'),
		love.graphics.newImage('GUI/HUD_C.png'),	
		love.graphics.newImage('GUI/GUI_HP.png'),		
	}

	--initializing font
	font = love.graphics.newFont("slkscre.ttf", 16)
	love.graphics.setFont(font)	
	
	--creating tileset
	TilesetPic = love.graphics.newImage('TILESET.png')
	Tileset = { }
	
	for j=1, TilesetX/TileSize do
		for i=1, TilesetY/TileSize do 
			Tileset[((i-1)*(TilesetX/TileSize))+j] = love.graphics.newQuad((j-1)*TileSize, (i-1)*TileSize, TileSize, TileSize, TilesetX, TilesetY)
		end
	end	
	
	--even player charset creation is here
	Charset_Creation()

	--creating Items-table
	ItemsPic = love.graphics.newImage('ITEMS.png')
	ItemsTileset = { }
	
	for j=1, TilesetX/TileSize do
		for i=1, TilesetY/TileSize do 
			ItemsTileset[((i-1)*(TilesetX/TileSize))+j] = love.graphics.newQuad((j-1)*TileSize, (i-1)*TileSize, TileSize, TileSize, TilesetX, TilesetY)
		end
	end	
	
	--uploading field
	Field = { }
	
	for j=1,SizeX do
	
		Field[j] = { }
	
		for i=1, SizeY do
			if math.random(100) < 2 then
				Field[j][i] = 18
				if i-1 ~= 0 and Field[j][i] ~= 2 then Field[j][i-1] = 2 end
			else
				Field[j][i] = 1			
			end
			
			if i==2 or j==2 or i==150 or j==150 then Field[j][i] = 2 end
		end
		
	end
	
	--defining player
	Player = {
	
	--moving and appearing
	X_Tile = OffsetX+(ScreenSizeX/2), Y_Tile = OffsetY+((ScreenSizeY-1)/2), 
	OffsetX = 0, OffsetY = 0,	
	Sprite = 1,
	Is_Moving = false, 
	Mov_Dir = "no",
	Facing = "right",
	
	In_Hand = 0,
	
	--stats
	HP = 100, Energy = 100,
	XP = 0, Lvl = 1,
	
	--abilities
	Moving_Speed = 3,						--moves 2^Mov_Spd pixels per frame		
	Inv_Capacity = 50,						--how much weight Player can carry on?
	
	--inventory
	Inventory = {}, 
	Hotbar = {0,0,0,0,0,0,0,0},
	Selected_Slot = 1
	}
	
	--inventory size = number of possible items
	for i=1, Item_Variety do
		table.insert(Player.Inventory, 0)
	end
	
	--defining characters
	Characters = {}

	CreateChar(2)	
	Characters[1].X_Tile = 26
	Characters[1].Y_Tile = 18
	Characters[1].State = 1
	Characters[1].Facing = "down"
	
	CreateChar(3)	
	Characters[2].X_Tile = 24
	Characters[2].Y_Tile = 18
	Characters[2].State = 1
	Characters[2].Facing = "down"	

	CreateChar(1)
	Characters[3].X_Tile = 28
	Characters[3].Y_Tile = 18	
	Characters[3].State = 2
	Characters[3].Facing = "down"	
	Characters[3].Mov_Dir = "up"		
	
	for z=4, 30 do
		CreateChar(3)
		Characters[z].X_Tile = 27+z
		Characters[z].Y_Tile = 19+z	
		Characters[z].State = 4
		Characters[z].Mov_Dir = MovSet[math.random(1,#MovSet)]
		Characters[z].Patrol_Distance = math.random(1,10)
	end

		
	--defining projectiles
	Projectiles = {}
	
	--defining items
	Items = {}
	
	for i=1, 15 do

		CreateItem(2)
		Items[i].X_Tile = (19+2*i)
		Items[i].Y_Tile = 19	
	
	end	
		CreateItem(1)
		Items[#Items].X_Tile = 21
		Items[#Items].Y_Tile = 22	
end





function love.draw()
	--appearing stuff; all these functions are explained in gui.lua
	Display_Textures()
	Display_HUD()

	if Inventory then Display_Inventory_Menu() end
	if DialogueActive then Display_Dialogue(Characters[Player_Neighbour()]) end
	
	--random text
	love.graphics.print(math.floor(FPS),0,16)
	
	if Player_Neighbour() ~= 0 
	and not DialogueActive 
	then love.graphics.print("Talk to "..Characters[Player_Neighbour()].Type,32,116) end
end




function love.update(dt)

	--key actions
	Repeated_Key_Behaviour() 

	--"constants"
	FPS = 1/dt
	
	MouseX, MouseY = love.mouse.getX(), love.mouse.getY()
	MouseTileX, MouseTileY = 1+math.floor((MouseX-OffsetX)/TileSize), 1+math.floor((MouseY-OffsetY)/TileSize)	
	
	if Inventory 
	and MouseX > InvX+32 and MouseY > InvY+32 
	and MouseX < 32+InvX+(InvColDist*InvColumnNum) and MouseY < 32+InvY+(InvColDist*InvColumnLength)
	and (MouseX-32-InvX)%InvColDist < TileSize and (MouseY-32-InvY)%InvRowDist < TileSize then
	
	--[[
	number of column: 1+math.floor((MouseX-32-InvX)/InvColDist)
	last element of columns: (1+math.floor((MouseX-32-InvX)/InvColDist))*InvColumnLength
	number of row: (1+math.floor((MouseY-32-InvY)/InvRowDist))
	]]
	
	InvSelectedItemID = (math.floor((MouseX-32-InvX)/InvColDist))*InvColumnLength + (1+math.floor((MouseY-32-InvY)/InvRowDist)) + InvListStart
	
	end
	
	--count inventory weight
	InvWeight = 0
	
	for i=1, #Player.Inventory do
		if ITEM_DOCUMENTATION[i] ~= nil then
			InvWeight = InvWeight + (Player.Inventory[i]*ITEM_DOCUMENTATION[i].Weight)
		end
	end
	
	--inhand item is always the selected hotbar item
	Player.In_Hand = Player.Hotbar[Player.Selected_Slot]
	
	--gameclock
	game_clock = game_clock + 1	
	
	--mouse detecting
	if MouseY < 50 and (MouseX < 250 or MouseX > 915) then
		MouseOnField = false
	elseif true then
		MouseOnField = true
	elseif true then
		MouseOnField = true
	else
		MouseOnField = true
	end
	
	--happens if certain menus are not opened
	if not Inventory and not DialogueActive then
	
		--turning
		WatchMousePlayer()
		
		--handle characters
		for i=#Characters, 1, -1 do
		
			--kill chars with 0 HP	
			if Characters[i].HP < 1 then Kill_Character(i) end
			
			--move all the chars
			if not Characters[i].Is_Moving then
				--phase 1: activate their behaviour
				if Characters[i].State == 2 then Patrol_Simple(Characters[i]) 
					if Characters[i].Weapon_ID ~= nil and game_clock%1 == 0 then Shoot_On_Sight(Characters[i]) end
				elseif Characters[i].State == 3 then Turning_Around(Characters[i])
				elseif Characters[i].State == 4 then Walking(Characters[i]) end				
			else
				--phase 2: smooth graphic moving
				SmoothMoving(false, Characters[i], Characters[i].Mov_Dir, Characters[i].Mov_Spd)			
			end	
			
			--move char face properly
			if Characters[i].State ~= 1 then FacingToMoving(Characters[i]) end

			--character animation
			Char_Anim(Characters[i])
			
		end		


		--doing smooth player moving 
		if Player.Is_Moving then
			SmoothMoving(true, Characters[1], Player.Mov_Dir, math.pow(2, Player.Moving_Speed))
			
			--item behaviour: pick items if standing above
			for i=#Items, 1, -1  do
				ItemBehaviour(i)
			end
			
		end
		
		BrokenProjectiles = {}
		
		--smooth moving of projectiles
		for i=1, #Projectiles do

			if #Projectiles ~= 0 then	
				SmoothMoving(false, Projectiles[i], Projectiles[i].Mov_Dir, Projectiles[i].Mov_Spd)
				
				--set orientation of projectiles
				if Projectiles[i].Mov_Dir == "down" or Projectiles[i].Mov_Dir == "up" then
					Projectiles[i].Sprite = 2
				elseif Projectiles[i].Mov_Dir == "left" or Projectiles[i].Mov_Dir == "right" then
					Projectiles[i].Sprite = 1				
				else
					Projectiles[i].Sprite = 3					
				end
				
				--checks if a projectile is already hit something
				Break_Projectile(Projectiles[i], i)
			end
			
			--adds the unused projectiles to a set
			if Projectiles[i].Mov_Dir == "DELETE ME" then table.insert(BrokenProjectiles, i) end

		end	
		
		--deleting unused projectiles
		for i=#BrokenProjectiles,1, -1 do
			table.remove(Projectiles, BrokenProjectiles[i])
		end
	
	end
end


--search integer in integer-table
function Search_in_table(int_table, number)
	
	local inIt = false
	
	for i=1, #int_table do
		if number == int_table[i] then	
			inIt = true
			break
		else
			inIt = false
		end
	end

		return inIt
end





































--possibly useful functions
function CountNeighbours(x,y, valueToCheck, Table)

	local asd=0

	if Table[x-1][y] == valueToCheck then asd=asd+1 end
	if Table[x+1][y] == valueToCheck then asd=asd+1 end
	if Table[x][y-1] == valueToCheck then asd=asd+1 end
	if Table[x][y+1] == valueToCheck then asd=asd+1 end	
	return asd
end

function FullNeighbours(x,y, valueToCheck, Table)
	local asd=0

	for j=-1,1 do
		for i=-1, 1 do
			if Table[x-j][y-i] == valueToCheck then asd=asd+1 end	
		end
	end
	
	return asd
end

function CountColors(Table, valueToCheck)

	local asd=0
	for j=2,127 do
		for i=2, 95 do
			if Table[j][i] == valueToCheck then asd=asd+1 end
		end
	end
	
	return asd
end

function ListColors(Table, valueToCheck)

	local list = {}
	for j=2,127 do
		for i=2, 95 do
			if Table[j][i] == valueToCheck then table.insert(list, {j,i}) end
		end
	end
	
	return list
end



function BackLoad(a,b)

	for j=1,128 do
		for i=1, 96 do
			a[j][i] = b[j][i]
		end
	end

end




